import 'base_unit.dart';
import 'prefixed_base_unit.dart';
import 'unit.dart';

NonDerivedUnit _mkStandard(BaseStandardUnit baseUnit) =>
    NonDerivedUnit(PrefixedBaseUnit(baseUnit));

NonDerivedUnit _mkNonStandard(
  String symbol,
  double factor,
  List<BaseStandardUnit> unitsUp, [
  List<BaseStandardUnit> unitsDown = const [],
]) =>
    NonDerivedUnit(
      PrefixedBaseUnit(BaseUnit(symbol, factor, unitsUp, unitsDown)),
    );

/// Base unit of length.
final meter = _mkStandard(meterBaseUnit);

/// Unit of length, `1 in. = 0.0254 m`.
final inch = _mkNonStandard('in', 0.0254, [meterBaseUnit]);

/// Unit of area.
final squareMeter = meter * meter;

/// Unit of volume, `1 L = 0.001 m^3`.
final liter =
    _mkNonStandard('L', 0.001, [meterBaseUnit, meterBaseUnit, meterBaseUnit]);

/// Unit of force, `1 N = 1 kg * m / s^2`.
final newton = _mkNonStandard(
  'N',
  1000,
  [gramBaseUnit, meterBaseUnit],
  [secondBaseUnit, secondBaseUnit],
);

/// Unit of pressure, `1 Pa = 1 N/m^2`.
final pascal = _mkNonStandard(
  'Pa',
  1000,
  [gramBaseUnit],
  [secondBaseUnit, secondBaseUnit, meterBaseUnit],
);

/// Unit of pressure, `1 cmH₂O = 98.0665 Pa`.
final cmH2O = _mkNonStandard(
  'cmH₂O',
  1000 * 98.0665,
  [gramBaseUnit],
  [secondBaseUnit, secondBaseUnit, meterBaseUnit],
);

/// Unit of pressure, `1 mmHg = 133.322387415 Pa`.
final mmHg = _mkNonStandard(
  'mmHg',
  1000 * 133.322387415,
  [gramBaseUnit],
  [secondBaseUnit, secondBaseUnit, meterBaseUnit],
);

/// Unit of pressure, `1 atm = 101325 Pa`.
final standardAtmosphere = _mkNonStandard(
  'atm',
  1000 * 101325,
  [gramBaseUnit],
  [secondBaseUnit, secondBaseUnit, meterBaseUnit],
);

/// Base unit of mass (note that the kilogram is the standard unit of mass in
/// SI, *NOT* the gram).
final gram = _mkStandard(gramBaseUnit);

/// Unit of mass, `1 lb = 453.59237 g`.
final pound = _mkNonStandard('lb', 453.59237, [gramBaseUnit]);

/// Base unit for amount of substance.
final mole = _mkStandard(moleBaseUnit);

/// Base unit of time.
final second = _mkStandard(secondBaseUnit);

const _secondsPerMinute = 60.0;

/// Unit of time, `1 min = 60 s`.
final minute = _mkNonStandard('min', _secondsPerMinute, [secondBaseUnit]);

const _secondsPerHour = _secondsPerMinute * 60;

/// Unit of time, `1 hr = 3600 s`.
final hour = _mkNonStandard('hr', _secondsPerHour, [secondBaseUnit]);

const _secondsPerDay = _secondsPerHour * 24;

/// Unit of time, `1 d = 24 h`.
final day = _mkNonStandard('d', _secondsPerDay, [secondBaseUnit]);

const _secondsPerWeek = _secondsPerDay * 7;

/// Unit of time, `1 wk = 7 d`.
final week = _mkNonStandard('wk', _secondsPerWeek, [secondBaseUnit]);

const _secondsPerMonth = _secondsPerDay * 30.436875;

/// Unit of time, `1 mo = 30.436875 d`.
final month = _mkNonStandard('mo', _secondsPerMonth, [secondBaseUnit]);

const _secondsPerYear = _secondsPerMonth * 12;

/// Unit of time, `1 yr = 12 mo`.
final year = _mkNonStandard('yr', _secondsPerYear, [secondBaseUnit]);

/// Base unit of temperature.
final kelvin = _mkStandard(kelvinBaseUnit);

/// Dimensionless unit, `1% = 1/100`.
final percent = _mkNonStandard('%', 1 / 100, const []);
